    # (Optional) Set this to the docker image that you want to use. If a requirements.txt is provided 
    # in your project directory, those requirements will be installed into this image.
    image: "nvcr.io/nvidia/pytorch:23.12-py3"

    workdir: "/app"

    # This is the directory that will be copied into your docker image before running it.
    # This isn't needed for the local backend, because you can mount in any directory you
    # want, instead of copying it into the docker image.
    workspace: null

    # Set the environment variables you want to use here.
    env:
      test_var: "test_value"

    # List of mounts, which are objects containing the keys "source", "target", and "type".
    # "source" is the source path of the mount
    # "target" is the mount point inside your docker container
    # "type" is one of ["bind", "volume", "device"]
    mounts:
      # mount the current project directory into /app
      - source: .
        target: /app
        type: bind
